@@ -8,14 +8,13 @@ use std::{
 };
 
 use anyhow::{anyhow, Context, Result};
-use bstr::{ByteSlice, ByteVec};
+use bstr::{BString, ByteSlice, ByteVec};
 use clap::{Arg, ArgGroup};
 
 use crate::{
     color::get_color_stdout,
-    patchedit,
-    patchname::PatchName,
-    signature::{self, SignatureExtended},
+    ext::{RepositoryExtended, TimeExtended},
+    patch::{patchedit, PatchName},
     stack::{InitializationPolicy, Stack, StackAccess, StackStateAccess},
     stupid::{Stupid, StupidContext},
 };
@@ -41,7 +40,7 @@ fn make() -> clap::Command {
              patch's author details.\n\
              \n\
              Patches may also be imported from a mail file (-m/--mail), an mbox \
-             (-M/--mbox), or a series (-s/--series). Furthermore, the -u/--url option \
+             (-M/--mbox), or a series (-S/--series). Furthermore, the -u/--url option \
              allows the patches source to be fetched from a url instead of from a \
              local file.\n\
              \n\
@@ -54,16 +53,16 @@ fn make() -> clap::Command {
             "stg import [OPTIONS] <diff-path>\n       \
              stg import [OPTIONS] -m [<mail-path>|<Maildir-path>]\n       \
              stg import [OPTIONS] -M [<mbox-path>]\n       \
-             stg import [OPTIONS] -s [<series-path>]\n       \
+             stg import [OPTIONS] -S [<series-path>]\n       \
              stg import [OPTIONS] -u <diff-url>\n       \
              stg import [OPTIONS] -u -m <mail-url>\n       \
              stg import [OPTIONS] -u -M <mbox-url>\n       \
-             stg import [OPTIONS] -u -s <series-url>"
+             stg import [OPTIONS] -u -S <series-url>"
         } else {
             "stg import [OPTIONS] <diff-path>\n       \
              stg import [OPTIONS] -m [<mail-path>|<Maildir-path>]\n       \
              stg import [OPTIONS] -M [<mbox-path>]\n       \
-             stg import [OPTIONS] -s [<series-path>]"
+             stg import [OPTIONS] -S [<series-path>]"
         })
         .arg(
             Arg::new("source")
@@ -94,7 +93,7 @@ fn make() -> clap::Command {
         .arg(
             Arg::new("series")
                 .long("series")
-                .short('s')
+                .short('S')
                 .help("Import patch series")
                 .long_help("Import patch series from a series file are tar archive.")
                 .action(clap::ArgAction::SetTrue),
@@ -217,8 +216,8 @@ fn make() -> clap::Command {
 }
 
 fn run(matches: &clap::ArgMatches) -> Result<()> {
-    let repo = git2::Repository::open_from_env()?;
-    let stack = Stack::from_branch(&repo, None, InitializationPolicy::AutoInitialize)?;
+    let repo = gix::Repository::open()?;
+    let stack = Stack::current(&repo, InitializationPolicy::AutoInitialize)?;
     let stupid = repo.stupid();
 
     let source_path = if matches.get_flag("url") {
@@ -230,8 +229,6 @@ fn run(matches: &clap::ArgMatches) -> Result<()> {
         None
     };
 
-    std::env::set_current_dir(repo.workdir().unwrap())?;
-
     let statuses = stupid.statuses(None)?;
     statuses.check_index_and_worktree_clean()?;
     stack.check_head_top_mismatch()?;
@@ -412,7 +409,7 @@ fn import_series(
                     Some(0)
                 } else {
                     return Err(anyhow!(
-                        "Patch `{}` has unsupported strip level \"{}\"",
+                        "patch `{}` has unsupported strip level \"{}\"",
                         raw_patchname.to_string_lossy(),
                         extra.to_str_lossy()
                     ));
@@ -442,19 +439,18 @@ fn find_series_path(base: &Path) -> Result<PathBuf> {
             return Ok(entry.path());
         }
     }
-    Err(anyhow!("Series file not found"))
+    Err(anyhow!("series file not found"))
 }
 
-fn use_message_id(matches: &clap::ArgMatches, config: &git2::Config) -> bool {
-    matches.get_flag("message-id") || config.get_bool("stgit.import.message-id").unwrap_or(false)
+fn use_message_id(matches: &clap::ArgMatches, config: &gix::config::Snapshot) -> bool {
+    matches.get_flag("message-id") || config.boolean("stgit.import.message-id").unwrap_or(false)
 }
 
 fn import_mail(stack: Stack, matches: &clap::ArgMatches, source_path: Option<&Path>) -> Result<()> {
     let out_dir = tempfile::tempdir()?;
     let missing_from_ok = matches.get_flag("mail");
     let keep_cr = matches.get_flag("keep-cr");
-    let config = stack.repo.config()?;
-    let message_id = use_message_id(matches, &config);
+    let message_id = use_message_id(matches, &stack.repo.config_snapshot());
     let stupid = stack.repo.stupid();
     let num_patches = stupid.mailsplit(source_path, out_dir.path(), keep_cr, missing_from_ok)?;
     let mut stack = stack;
@@ -516,8 +512,7 @@ fn import_file<'repo>(
     source_path: Option<&Path>,
     strip_level: Option<usize>,
 ) -> Result<Stack<'repo>> {
-    let config = stack.repo.config()?;
-    let message_id = use_message_id(matches, &config);
+    let message_id = use_message_id(matches, &stack.repo.config_snapshot());
     let stupid = stack.repo.stupid();
 
     let (mailinfo, message, diff) = if let Some(source_path) = source_path {
@@ -569,6 +564,8 @@ fn create_patch<'repo>(
     diff: &[u8],
     strip_level: Option<usize>,
 ) -> Result<Stack<'repo>> {
+    let config = stack.repo.config_snapshot();
+
     let Headers {
         patchname,
         author_name,
@@ -601,7 +598,6 @@ fn create_patch<'repo>(
         patchname
     };
 
-    let config = stack.repo.config()?;
     let name_len_limit = PatchName::get_length_limit(&config);
 
     let patchname = if let Some(patchname) = patchname {
@@ -623,20 +619,26 @@ fn create_patch<'repo>(
         patchname
     };
 
-    let author_date = author_date.and_then(|date| crate::signature::parse_time(&date).ok());
+    let author_date = author_date.and_then(|date| gix::actor::Time::parse_time(&date).ok());
     let author = if let (Some(name), Some(email), Some(time)) =
-        (author_name.as_ref(), author_email.as_ref(), author_date)
+        (author_name.as_deref(), author_email.as_deref(), author_date)
     {
-        git2::Signature::new(name, email, &time)?
-    } else if let (Some(name), Some(email)) = (author_name.as_ref(), author_email.as_ref()) {
-        git2::Signature::now(name, email)?
-    } else if let Some(time) = author_date {
-        let default = git2::Signature::default_author(Some(&config))?;
-        let name = default.name().expect("default author name is UTF-8");
-        let email = default.email().expect("default author email is UTF-8");
-        git2::Signature::new(name, email, &time)?
+        gix::actor::Signature {
+            name: BString::from(name),
+            email: BString::from(email),
+            time,
+        }
     } else {
-        git2::Signature::default_author(Some(&config))?
+        let default_author = stack.repo.get_author()?;
+        if let (Some(name), Some(email)) = (author_name.as_deref(), author_email.as_deref()) {
+            gix::actor::Signature {
+                name: BString::from(name),
+                email: BString::from(email),
+                time: default_author.time,
+            }
+        } else {
+            default_author.to_owned()
+        }
     };
 
     let strip_level = strip_level.or_else(|| matches.get_one::<usize>("strip").copied());
@@ -644,7 +646,7 @@ fn create_patch<'repo>(
     let trimmed_diff = diff.trim_end_with(|c| c.is_ascii_whitespace());
 
     let tree_id = if trimmed_diff.is_empty() || trimmed_diff == b"---" {
-        stack.get_branch_head().tree_id()
+        stack.get_branch_head().tree_id()?.detach()
     } else {
         let stupid = stack.repo.stupid();
         stupid.apply_to_worktree_and_index(
@@ -661,10 +663,10 @@ fn create_patch<'repo>(
         stupid.write_tree()?
     };
 
-    let (new_patchname, commit_id) = match crate::patchedit::EditBuilder::default()
+    let (new_patchname, commit_id) = match crate::patch::edit::EditBuilder::default()
         .original_patchname(Some(&patchname))
         .override_tree_id(tree_id)
-        .override_parent_id(stack.get_branch_head().id())
+        .override_parent_id(stack.get_branch_head().id)
         .default_author(author)
         .default_message(message)
         .allow_autosign(true)
@@ -781,7 +783,7 @@ impl Headers {
                     headers.patchname = Some(
                         value
                             .to_str()
-                            .map_err(|_| anyhow!("Patch name is not UTF-8"))
+                            .map_err(|_| anyhow!("patch name is not UTF-8"))
                             .context("parsing Patch header")?
                             .to_string(),
                     );
@@ -792,7 +794,7 @@ impl Headers {
                     let (name, email) = value
                         .to_str()
                         .map_err(|_| anyhow!("From/Author is not UTF-8"))
-                        .and_then(signature::parse_name_email)
+                        .and_then(patchedit::parse_name_email)
                         .context("parsing From/Author header")?;
                     headers.author_name = Some(name.to_string());
                     headers.author_email = Some(email.to_string());
@@ -820,7 +822,7 @@ impl Headers {
             // Once the subject is determined, more contiguous headers may be parsed.
             headers.subject = Some(
                 line.to_str()
-                    .map_err(|_| anyhow!("Message is not UTF-8"))
+                    .map_err(|_| anyhow!("message is not UTF-8"))
                     .context("parsing patch message")?
                     .to_string(),
             );
