@@ -5,14 +5,15 @@
 use std::{io::Write, str::FromStr};
 
 use anyhow::{anyhow, Result};
+use bstr::ByteSlice;
 use clap::{Arg, ArgGroup, ArgMatches, ValueHint};
 use termcolor::WriteColor;
 
 use crate::{
     argset,
-    commit::CommitExtended,
-    patchname::PatchName,
-    patchrange,
+    branchloc::BranchLocator,
+    ext::{CommitExtended, RepositoryExtended},
+    patch::{patchrange, PatchName, PatchRange, RangeConstraint},
     stack::{InitializationPolicy, Stack, StackAccess, StackStateAccess},
 };
 
@@ -29,15 +30,18 @@ fn make() -> clap::Command {
     clap::Command::new(STGIT_COMMAND.name)
         .about("Display the patch series")
         .long_about(
-            "Show all the patches in the series, or just those in the \
-             given range, ordered from top to bottom.\n\
+            "Show all the patches in the series, or just those in the given range, \
+             ordered from bottom to top.\n\
              \n\
-             The applied patches are prefixed with a '+' (except the \
-             current patch, which is prefixed with a '>'), the \
-             unapplied patches with a '-', and the hidden patches with \
-             a '!'.\n\
+             The topmost applied patch is prefixed with '>'. All other applied patches \
+             are prefixed with '+'. Unapplied patches are prefixed with '-' and hidden \
+             patches with '!'.\n\
              \n\
-             Empty patches are prefixed with a '0'.",
+             The --reverse option may be used to reverse the order in which patches \
+             are displayed. The reversed order is more stack-like, with the base of \
+             the stack appearing at the bottom of of the display.\n\
+             \n\
+             Empty patches are prefixed with a '*' when the --empty option is used.",
         )
         .override_usage(
             "stg series [OPTIONS] [-A] [-U] [-H]\n       \
@@ -50,7 +54,8 @@ fn make() -> clap::Command {
                 .help("Patches to display")
                 .value_name("patch")
                 .num_args(1..)
-                .value_parser(clap::value_parser!(patchrange::Specification))
+                .allow_hyphen_values(true)
+                .value_parser(clap::value_parser!(PatchRange))
                 .conflicts_with_all(["all", "applied", "unapplied", "hidden", "short"]),
         )
         .arg(argset::branch_arg())
@@ -67,8 +72,11 @@ fn make() -> clap::Command {
             Arg::new("short")
                 .long("short")
                 .short('s')
-                .help("Select patches around the topmost patch only")
-                .action(clap::ArgAction::SetTrue),
+                .help("Select <n> patches around the topmost patch only")
+                .value_name("n")
+                .num_args(0..=1)
+                .require_equals(true)
+                .value_parser(argset::parse_usize),
         )
         .group(ArgGroup::new("all-short-group").args(["all", "short"]))
         .arg(
@@ -99,6 +107,7 @@ fn make() -> clap::Command {
                 .help("Select patches in <branch> not present in current branch")
                 .num_args(1)
                 .value_name("branch")
+                .value_parser(clap::value_parser!(BranchLocator))
                 .value_hint(ValueHint::Other),
         )
         .next_help_heading("Display Options")
@@ -108,6 +117,14 @@ fn make() -> clap::Command {
                 .help("Display author name for each patch")
                 .action(clap::ArgAction::SetTrue),
         )
+        .arg(
+            Arg::new("no-author")
+                .long("no-author")
+                .help("Do not display patch authors")
+                .hide(true)
+                .action(clap::ArgAction::SetTrue)
+                .overrides_with("author"),
+        )
         .arg(
             Arg::new("count")
                 .long("count")
@@ -135,6 +152,14 @@ fn make() -> clap::Command {
                 .require_equals(true)
                 .value_parser(clap::value_parser!(CommitIdLength)),
         )
+        .arg(
+            Arg::new("no-commit-id")
+                .long("no-commit-id")
+                .help("Do not display commit ids")
+                .hide(true)
+                .action(clap::ArgAction::SetTrue)
+                .overrides_with("commit-id"),
+        )
         .arg(
             Arg::new("description")
                 .long("description")
@@ -147,10 +172,10 @@ fn make() -> clap::Command {
             Arg::new("no-description")
                 .long("no-description")
                 .help("Do not display the patch description")
+                .hide(true)
                 .action(clap::ArgAction::SetTrue)
                 .overrides_with("description"),
         )
-        .group(ArgGroup::new("description-group").args(["description", "no-description"]))
         .arg(
             Arg::new("empty")
                 .long("empty")
@@ -163,6 +188,22 @@ fn make() -> clap::Command {
                 )
                 .action(clap::ArgAction::SetTrue),
         )
+        .arg(
+            Arg::new("no-empty")
+                .long("no-empty")
+                .help("Do not display whether patches are empty")
+                .hide(true)
+                .action(clap::ArgAction::SetTrue)
+                .overrides_with("empty"),
+        )
+        .arg(
+            Arg::new("prefix")
+                .long("prefix")
+                .help("Display patch status prefixes (default)")
+                .hide(true)
+                .action(clap::ArgAction::SetTrue)
+                .overrides_with("no-prefix"),
+        )
         .arg(
             Arg::new("no-prefix")
                 .long("no-prefix")
@@ -171,12 +212,80 @@ fn make() -> clap::Command {
                 .help("Do not display the patch status prefix")
                 .action(clap::ArgAction::SetTrue),
         )
+        .arg(
+            Arg::new("indices")
+                .long("indices")
+                .short('I')
+                .help("Display absolute patch indicies")
+                .action(clap::ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new("no-indices")
+                .long("no-indices")
+                .help("Do not display patch indices")
+                .hide(true)
+                .action(clap::ArgAction::SetTrue)
+                .overrides_with("indices"),
+        )
+        .arg(
+            Arg::new("offsets")
+                .long("offsets")
+                .short('O')
+                .help("Display relative offsets from topmost patch")
+                .long_help(
+                    "Display relative offsets from topmost patch or from the stack \
+                     base if no patches are applied.",
+                )
+                .action(clap::ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new("no-offsets")
+                .long("no-offsets")
+                .help("Do not display patch offsets")
+                .hide(true)
+                .action(clap::ArgAction::SetTrue)
+                .overrides_with("offsets"),
+        )
+        .arg(
+            Arg::new("reverse")
+                .long("reverse")
+                .short('r')
+                .help("Display patches in reverse order")
+                .long_help(
+                    "Display patches in reverse order.\n\
+                     \n\
+                     This causes the stack to be displayed \"right-side up\". By \
+                     default, patches are printed in order from bottom to top; i.e. \
+                     the first patch in the stack is printed first and the last patch \
+                     is printed last. Thus with the default ordering, the stack is \
+                     displayed upside down. Reversing the order flips the stack such \
+                     that the bottom of the stack is spatially at the top of the \
+                     display, which may be more intuitive.",
+                )
+                .action(clap::ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new("no-reverse")
+                .long("no-reverse")
+                .help("Do not display patches in reverse order")
+                .hide(true)
+                .action(clap::ArgAction::SetTrue)
+                .overrides_with("reverse"),
+        )
         .arg(
             Arg::new("show-branch")
                 .long("showbranch")
                 .help("Display the branch name with the listed patches")
                 .action(clap::ArgAction::SetTrue),
         )
+        .arg(
+            Arg::new("no-show-branch")
+                .long("no-showbranch")
+                .help("Do not display branch name")
+                .hide(true)
+                .action(clap::ArgAction::SetTrue)
+                .overrides_with("show-branch"),
+        )
 }
 
 #[derive(Clone)]
@@ -208,18 +317,18 @@ impl FromStr for CommitIdLength {
 }
 
 fn run(matches: &ArgMatches) -> Result<()> {
-    let repo = git2::Repository::open_from_env()?;
-    let opt_branch = argset::get_one_str(matches, "branch");
-    let opt_missing = argset::get_one_str(matches, "missing");
+    let repo = gix::Repository::open()?;
+    let opt_branch = matches.get_one::<BranchLocator>("branch");
+    let opt_missing = matches.get_one::<BranchLocator>("missing");
 
     let (stack, ref_stack) = if let Some(ref_branch) = opt_missing {
         (
-            Stack::from_branch(
+            Stack::from_branch_locator(
                 &repo,
                 Some(ref_branch),
                 InitializationPolicy::AllowUninitialized,
             )?,
-            Some(Stack::from_branch(
+            Some(Stack::from_branch_locator(
                 &repo,
                 opt_branch,
                 InitializationPolicy::RequireInitialized,
@@ -227,7 +336,11 @@ fn run(matches: &ArgMatches) -> Result<()> {
         )
     } else {
         (
-            Stack::from_branch(&repo, opt_branch, InitializationPolicy::AllowUninitialized)?,
+            Stack::from_branch_locator(
+                &repo,
+                opt_branch,
+                InitializationPolicy::AllowUninitialized,
+            )?,
             None,
         )
     };
@@ -237,16 +350,24 @@ fn run(matches: &ArgMatches) -> Result<()> {
     let unapplied_flag = matches.get_flag("unapplied");
     let hidden_flag = matches.get_flag("hidden");
 
-    let mut patches: Vec<(PatchName, git2::Oid, char)> = vec![];
+    struct Entry {
+        patchname: PatchName,
+        commit_id: gix::ObjectId,
+        sigil: char,
+        index: usize,
+        offset_from_top: isize,
+    }
 
-    if let Some(range_specs) = matches.get_many::<patchrange::Specification>("patchranges-all") {
+    let mut patches: Vec<Entry> = vec![];
+
+    if let Some(range_specs) = matches.get_many::<PatchRange>("patchranges-all") {
         let top_patchname = stack.applied().last();
-        for patchname in patchrange::contiguous_patches_from_specs(
-            range_specs,
+        for patchname in patchrange::resolve_names_contiguous(
             &stack,
-            patchrange::Allow::AllWithAppliedBoundary,
+            range_specs,
+            RangeConstraint::AllWithAppliedBoundary,
         )? {
-            let commit_id = stack.get_patch_commit(&patchname).id();
+            let commit_id = stack.get_patch_commit_id(&patchname);
             let sigil = if Some(&patchname) == top_patchname {
                 '>'
             } else if stack.is_applied(&patchname) {
@@ -256,7 +377,15 @@ fn run(matches: &ArgMatches) -> Result<()> {
             } else {
                 '!'
             };
-            patches.push((patchname, commit_id, sigil));
+            let index = stack.index_of(&patchname);
+            let offset_from_top = stack.distance_from(&patchname, top_patchname);
+            patches.push(Entry {
+                patchname,
+                commit_id,
+                sigil,
+                index,
+                offset_from_top,
+            });
         }
     } else {
         let show_applied = applied_flag || all_flag || !(unapplied_flag || hidden_flag);
@@ -266,45 +395,70 @@ fn run(matches: &ArgMatches) -> Result<()> {
         if show_applied {
             if let Some((last_patchname, rest)) = stack.applied().split_last() {
                 for patchname in rest {
-                    let commit_id = stack.get_patch_commit(patchname).id();
-                    patches.push((patchname.clone(), commit_id, '+'));
+                    patches.push(Entry {
+                        patchname: patchname.clone(),
+                        commit_id: stack.get_patch_commit_id(patchname),
+                        sigil: '+',
+                        index: stack.index_of(patchname),
+                        offset_from_top: stack.distance_from(patchname, Some(last_patchname)),
+                    });
                 }
-                let last_oid = stack.get_patch_commit(last_patchname).id();
-                patches.push((last_patchname.clone(), last_oid, '>'));
+                patches.push(Entry {
+                    patchname: last_patchname.clone(),
+                    commit_id: stack.get_patch_commit_id(last_patchname),
+                    sigil: '>',
+                    index: stack.index_of(last_patchname),
+                    offset_from_top: 0,
+                });
             }
         }
 
         if show_unapplied {
-            for patchname in stack.unapplied() {
-                let commit_id = stack.get_patch_commit(patchname).id();
-                patches.push((patchname.clone(), commit_id, '-'));
+            for (i, patchname) in stack.unapplied().iter().enumerate() {
+                patches.push(Entry {
+                    patchname: patchname.clone(),
+                    commit_id: stack.get_patch_commit_id(patchname),
+                    sigil: '-',
+                    index: stack.index_of(patchname),
+                    offset_from_top: (i + 1) as isize,
+                });
             }
         }
 
         if show_hidden {
+            let top_patchname = stack.applied().last();
             for patchname in stack.hidden() {
-                let commit_id = stack.get_patch_commit(patchname).id();
-                patches.push((patchname.clone(), commit_id, '!'));
+                patches.push(Entry {
+                    patchname: patchname.clone(),
+                    commit_id: stack.get_patch_commit_id(patchname),
+                    sigil: '!',
+                    index: stack.index_of(patchname),
+                    offset_from_top: stack.distance_from(patchname, top_patchname),
+                });
             }
         }
     }
 
     if let Some(ref_stack) = ref_stack {
-        patches.retain(|(patchname, _, _)| {
+        patches.retain(|Entry { patchname, .. }| {
             ref_stack
                 .all_patches()
                 .all(|ref_patchname| patchname != ref_patchname)
         });
     }
 
-    if matches.get_flag("short") {
-        let shortnr = repo
-            .config()
-            .and_then(|config| config.get_i32("stgit.shortnr"))
-            .unwrap_or(5);
-        let shortnr: usize = if shortnr < 0 { 0 } else { shortnr as usize };
+    if matches.contains_id("short") {
+        let shortnr = matches
+            .get_one::<usize>("short")
+            .copied()
+            .unwrap_or_else(|| {
+                repo.config_snapshot()
+                    .integer("stgit.shortnr")
+                    .map(|i| if i.is_negative() { 0 } else { i as usize })
+                    .unwrap_or(5)
+            });
 
-        if let Some(top_pos) = patches.iter().position(|(_, _, sigil)| *sigil == '>') {
+        if let Some(top_pos) = patches.iter().position(|Entry { sigil, .. }| *sigil == '>') {
             if patches.len() - top_pos > shortnr {
                 patches.drain(top_pos + 1 + shortnr..);
             }
@@ -333,7 +487,11 @@ fn run(matches: &ArgMatches) -> Result<()> {
     };
 
     let patchname_width = if opt_commit_id.is_some() || description_flag || author_flag {
-        patches.iter().map(|(pn, _, _)| pn.len()).max().unwrap_or(0)
+        patches
+            .iter()
+            .map(|Entry { patchname, .. }| patchname.len())
+            .max()
+            .unwrap_or(0)
     } else {
         0
     };
@@ -341,10 +499,14 @@ fn run(matches: &ArgMatches) -> Result<()> {
     let author_width: usize = if author_flag && description_flag {
         patches
             .iter()
-            .map(|(_, commit_id, _)| -> usize {
+            .map(|Entry { commit_id, .. }| -> usize {
                 if let Ok(commit) = repo.find_commit(*commit_id) {
-                    let author = commit.author();
-                    author.name().unwrap_or(UNPRINTABLE).len()
+                    commit
+                        .author()
+                        .ok()
+                        .and_then(|author| author.name.to_str().ok())
+                        .unwrap_or(UNPRINTABLE)
+                        .len()
                 } else {
                     0
                 }
@@ -357,36 +519,77 @@ fn run(matches: &ArgMatches) -> Result<()> {
 
     let no_prefix_flag = matches.get_flag("no-prefix");
     let empty_flag = matches.get_flag("empty");
+    let indices_flag = matches.get_flag("indices");
+    let offsets_flag = matches.get_flag("offsets");
+
+    let index_width = (indices_flag && !patches.is_empty())
+        .then(|| patches.last().unwrap().index.to_string().len())
+        .unwrap_or_default();
+
+    let offset_width = (offsets_flag && !patches.is_empty())
+        .then(|| {
+            [patches.first().unwrap(), patches.last().unwrap()]
+                .iter()
+                .map(|entry| format!("{:+}", entry.offset_from_top).len())
+                .max()
+                .unwrap()
+        })
+        .unwrap_or_default();
 
     let mut stdout = crate::color::get_color_stdout(matches);
     let mut color_spec = termcolor::ColorSpec::new();
 
-    for (patchname, commit_id, sigil) in patches {
+    if matches.get_flag("reverse") {
+        patches.reverse();
+    }
+
+    for Entry {
+        patchname,
+        commit_id,
+        sigil,
+        index,
+        offset_from_top,
+    } in patches
+    {
         let commit = repo.find_commit(commit_id)?;
+        let commit_ref = commit.decode()?;
 
         if empty_flag {
             if commit.is_no_change()? {
                 stdout.set_color(color_spec.set_fg(Some(termcolor::Color::Cyan)))?;
-                write!(stdout, "0")?;
+                write!(stdout, "*")?;
                 stdout.set_color(color_spec.set_fg(None))?;
             } else {
                 write!(stdout, " ")?;
             }
         }
 
+        let sigil_color = match sigil {
+            '+' => Some(termcolor::Color::Green),
+            '>' => Some(termcolor::Color::Blue),
+            '-' => Some(termcolor::Color::Magenta),
+            '!' => Some(termcolor::Color::Red),
+            _ => None,
+        };
+
         if !no_prefix_flag {
-            let sigil_color = match sigil {
-                '+' => Some(termcolor::Color::Green),
-                '>' => Some(termcolor::Color::Blue),
-                '-' => Some(termcolor::Color::Magenta),
-                '!' => Some(termcolor::Color::Red),
-                _ => None,
-            };
             stdout.set_color(color_spec.set_fg(sigil_color))?;
             write!(stdout, "{sigil} ")?;
             stdout.set_color(color_spec.set_fg(None))?;
         }
 
+        if indices_flag {
+            stdout.set_color(color_spec.set_fg(sigil_color))?;
+            write!(stdout, "{index:index_width$} ")?;
+            stdout.set_color(color_spec.set_fg(None))?;
+        }
+
+        if offsets_flag {
+            stdout.set_color(color_spec.set_fg(sigil_color))?;
+            write!(stdout, "{offset_from_top:+offset_width$} ")?;
+            stdout.set_color(color_spec.set_fg(None))?;
+        }
+
         match sigil {
             '+' => color_spec.set_intense(true),
             '>' => color_spec.set_bold(true),
@@ -417,19 +620,21 @@ fn run(matches: &ArgMatches) -> Result<()> {
             write!(stdout, " # ")?;
             stdout.set_color(color_spec.set_fg(Some(termcolor::Color::Blue)))?;
             if let Ok(author) = commit.author_strict() {
-                write!(stdout, "{:author_width$}", &author.name().unwrap(),)?;
+                write!(stdout, "{:author_width$}", &author.name.to_str().unwrap())?;
             } else {
-                let author = commit.author();
-                let name = String::from_utf8_lossy(author.name_bytes());
+                let name = commit_ref.author().name.to_str_lossy();
                 write!(stdout, "{name:author_width$}")?;
             }
         }
         if description_flag {
             stdout.set_color(color_spec.set_fg(Some(termcolor::Color::Black)))?;
             write!(stdout, " #")?;
-            if let Some(summary) = commit.summary() {
-                stdout.set_color(color_spec.set_fg(None))?;
-                write!(stdout, " {summary}")?;
+            let summary = commit_ref.message_summary();
+            if !summary.is_empty() {
+                if let Ok(summary) = summary.to_str() {
+                    stdout.set_color(color_spec.set_fg(None))?;
+                    write!(stdout, " {summary}")?;
+                }
             }
         }
         color_spec.clear();
