@@ -2,27 +2,32 @@
 
 //! Support for using git repository hooks.
 
-use std::{io::Write, path::PathBuf};
+use std::{borrow::Cow, io::Write, path::PathBuf};
 
 use anyhow::{anyhow, Context, Result};
 
-use crate::commit::CommitMessage;
+use crate::wrap::Message;
 
 /// Find path to hook script given a hook name.
-fn get_hook_path(repo: &git2::Repository, hook_name: &str) -> PathBuf {
-    let hooks_path = if let Ok(config) = repo.config() {
-        config
-            .get_path("core.hookspath")
-            .unwrap_or_else(|_| PathBuf::from("hooks"))
-    } else {
-        PathBuf::from("hooks")
-    };
-    let hooks_root = if hooks_path.is_absolute() {
-        hooks_path
-    } else {
-        repo.path().join(hooks_path)
-    };
-    hooks_root.join(hook_name)
+fn get_hook_path(repo: &gix::Repository, hook_name: &str) -> Result<PathBuf> {
+    let config = repo.config_snapshot();
+    let hooks_path =
+        if let Some(core_hooks_path) = config.trusted_path("core.hookspath").transpose()? {
+            if core_hooks_path.is_absolute() {
+                core_hooks_path
+            } else if repo.is_bare() {
+                // The hooks path is relative to GIT_DIR in the case of a bare repo
+                Cow::Owned(repo.git_dir().join(core_hooks_path))
+            } else {
+                // The hooks path is relative to the root of the working tree otherwise
+                let work_dir = repo.work_dir().expect("non-bare repo must have work dir");
+                Cow::Owned(work_dir.join(core_hooks_path))
+            }
+        } else {
+            // No core.hookspath, use default .git/hooks location
+            Cow::Owned(repo.git_dir().join("hooks"))
+        };
+    Ok(hooks_path.join(hook_name))
 }
 
 /// Run the git `pre-commit` hook script.
@@ -33,9 +38,9 @@ fn get_hook_path(repo: &git2::Repository, hook_name: &str) -> PathBuf {
 /// Returns `Ok(true)` if the hook ran and completed successfully, `Err()` if the hook ran but failed,
 /// and `Ok(false)` if the hook did not run due to the script not existing, not being a file, or not
 /// being executable.
-pub(crate) fn run_pre_commit_hook(repo: &git2::Repository, use_editor: bool) -> Result<bool> {
+pub(crate) fn run_pre_commit_hook(repo: &gix::Repository, use_editor: bool) -> Result<bool> {
     let hook_name = "pre-commit";
-    let hook_path = get_hook_path(repo, hook_name);
+    let hook_path = get_hook_path(repo, hook_name)?;
     let hook_meta = match std::fs::metadata(&hook_path) {
         Ok(meta) => meta,
         Err(_) => return Ok(false), // ignore missing hook
@@ -52,7 +57,7 @@ pub(crate) fn run_pre_commit_hook(repo: &git2::Repository, use_editor: bool) ->
 
     let mut hook_command = std::process::Command::new(hook_path);
     let workdir = repo
-        .workdir()
+        .work_dir()
         .expect("should not get this far with a bare repo");
     if !use_editor {
         hook_command.env("GIT_EDITOR", ":");
@@ -85,12 +90,12 @@ pub(crate) fn run_pre_commit_hook(repo: &git2::Repository, use_editor: bool) ->
 /// Returns successfully if the hook script does not exist, is not a file, or is not
 /// executable.
 pub(crate) fn run_commit_msg_hook<'repo>(
-    repo: &git2::Repository,
-    message: CommitMessage<'repo>,
+    repo: &gix::Repository,
+    message: Message<'repo>,
     use_editor: bool,
-) -> Result<CommitMessage<'repo>> {
+) -> Result<Message<'repo>> {
     let hook_name = "commit-msg";
-    let hook_path = get_hook_path(repo, hook_name);
+    let hook_path = get_hook_path(repo, hook_name)?;
     let hook_meta = match std::fs::metadata(&hook_path) {
         Ok(meta) => meta,
         Err(_) => return Ok(message), // ignore missing hook
@@ -109,13 +114,12 @@ pub(crate) fn run_commit_msg_hook<'repo>(
     msg_file.write_all(message.raw_bytes())?;
     let msg_file_path = msg_file.into_temp_path();
 
-    let index = repo.index()?;
-    let index_path = index.path().expect("repo's default index must be a file");
+    let index_path = repo.index_path();
 
     // TODO: when git runs this hook, it only sets GIT_INDEX_FILE and sometimes
     // GIT_EDITOR. So author and committer vars are not clearly required.
-    let mut hook_command = std::process::Command::new(hook_path);
-    hook_command.env("GIT_INDEX_FILE", index_path);
+    let mut hook_command = std::process::Command::new(&hook_path);
+    hook_command.env("GIT_INDEX_FILE", &index_path);
     if !use_editor {
         hook_command.env("GIT_EDITOR", ":");
     }
@@ -132,10 +136,10 @@ pub(crate) fn run_commit_msg_hook<'repo>(
         let message = encoding
             .decode_without_bom_handling_and_without_replacement(&message_bytes)
             .ok_or_else(|| {
-                anyhow!("Message could not be decoded with `{}`", encoding.name())
+                anyhow!("message could not be decoded with `{}`", encoding.name())
                     .context("`{hook_name}` hook")
             })?;
-        Ok(CommitMessage::from(message.to_string()))
+        Ok(Message::from(message.to_string()))
     } else {
         Err(anyhow!(
             "`{hook_name}` hook returned {}",
