@@ -9,25 +9,24 @@
 
 mod alias;
 mod argset;
+mod branchloc;
 mod cmd;
 mod color;
-mod commit;
+mod ext;
 mod hook;
-mod patchedit;
-mod patchname;
-mod patchrange;
-mod repo;
-mod revspec;
+mod patch;
 mod signal;
-mod signature;
 mod stack;
 mod stupid;
 mod templates;
+mod wrap;
 
 use std::{ffi::OsString, io::Write, path::PathBuf};
 
 use anyhow::{anyhow, Context, Result};
+use bstr::ByteSlice;
 use clap::ArgMatches;
+use ext::RepositoryExtended;
 use stupid::StupidContext;
 use termcolor::WriteColor;
 
@@ -139,6 +138,10 @@ pub(crate) fn get_full_command(
 /// quickly as possible.
 fn main() -> ! {
     let argv: Vec<OsString> = std::env::args_os().collect();
+    unsafe {
+        // SAFETY: It's sound as we don't manipulate environment variables while querying local offsets.
+        time::util::local_offset::set_soundness(time::util::local_offset::Soundness::Unsound);
+    }
 
     // Chicken and egg: the --color option must be parsed from argv in order to setup
     // clap with the desired color choice. So a simple pre-parse is performed just to
@@ -168,11 +171,6 @@ fn main() -> ! {
         } else if matches.get_flag("help-option") {
             full_app_help(argv, None, color_choice)
         } else if let Some((sub_name, sub_matches)) = matches.subcommand() {
-            // Initialize git2 library only in case a subcommand is matched. This limits
-            // the cost of libgit2 initialization to code paths that will actually use
-            // libgit2.
-            unsafe { init_libgit2_extensions() }.expect("can set libgit2 extensions");
-
             // If the name matches any known subcommands, then only the Command for that
             // particular command is constructed and the costs of searching for aliases
             // and constructing all subcommands' Command instances are avoided.
@@ -232,34 +230,6 @@ fn main() -> ! {
     }
 }
 
-/// Initialize [`git2`] library extensions.
-///
-/// By default, [`git2`] fails any operation if it encounters any unknown extensions
-/// configured in a repo. StGit is uses [`git2`] library as an interface to the git
-/// object database and for interrogating the git configuration.
-///
-/// As a workaround until `worktreeconfig` extension is supported upstream, add it to
-/// whitelisted extensions.
-///
-/// # Safety
-/// * Modifying whitelisted extensions in [`git2`] is like modifying a static mut
-///   variable (unsafe).
-/// * This function must be called at initialization time (before git2 library usage)
-///   and should be called only once.
-unsafe fn init_libgit2_extensions() -> Result<()> {
-    git2::opts::set_extensions(&["worktreeconfig"]).context("set `worktreeconfig` extension")
-}
-
-#[cfg(test)]
-mod test {
-    use super::*;
-
-    #[test]
-    fn test_libgit2_extensions() {
-        unsafe { init_libgit2_extensions() }.unwrap();
-    }
-}
-
 /// Exit the program based on the provided [`Result`].
 ///
 /// Error results from conflicts trigger merge conflicts to be printed and an exit code
@@ -268,17 +238,36 @@ fn exit_with_result(result: Result<()>, color_choice: Option<termcolor::ColorCho
     let code = match result {
         Ok(()) => 0,
         Err(e) => {
+            // A command may use a custom clap error when doing argument validation after
+            // calling Command::try_get_matches_from().
+            if let Some(clap_err) = e.downcast_ref::<clap::Error>() {
+                clap_err.print().expect("clap can print its error message");
+                std::process::exit(if clap_err.use_stderr() {
+                    GENERAL_ERROR
+                } else {
+                    0
+                })
+            }
+
             print_error_message(color_choice, &e);
-            match e.downcast_ref::<stack::Error>() {
-                Some(stack::Error::TransactionHalt { conflicts, .. }) => {
-                    if *conflicts {
-                        print_merge_conflicts();
+
+            if let Some(e) = e.downcast_ref::<stack::TransactionError>() {
+                match e {
+                    stack::TransactionError::TransactionHalt { conflicts, .. } => {
+                        if *conflicts {
+                            print_merge_conflicts();
+                        }
+                        CONFLICT_ERROR
                     }
-                    CONFLICT_ERROR
+                    stack::TransactionError::CheckoutConflicts(_) => CONFLICT_ERROR,
                 }
-                Some(stack::Error::CheckoutConflicts(_))
-                | Some(stack::Error::CausedConflicts(_)) => CONFLICT_ERROR,
-                _ => COMMAND_ERROR,
+            } else if let Some(e) = e.downcast_ref::<cmd::Error>() {
+                match e {
+                    cmd::Error::CausedConflicts(_) => CONFLICT_ERROR,
+                    _ => COMMAND_ERROR,
+                }
+            } else {
+                COMMAND_ERROR
             }
         }
     };
@@ -366,7 +355,7 @@ fn execute_shell_alias(
     alias: &alias::Alias,
     user_args: Vec<OsString>,
     color_choice: Option<termcolor::ColorChoice>,
-    repo: Option<&git2::Repository>,
+    repo: Option<&gix::Repository>,
 ) -> ! {
     if let Some(first_arg) = user_args.first() {
         if [OsString::from("-h"), OsString::from("--help")].contains(first_arg) {
@@ -376,12 +365,9 @@ fn execute_shell_alias(
 
     // TODO: Git chooses its shell path at compile time based on OS or user override.
     let shell_path = "sh";
-    let shell_chars = &[
-        '|', '&', ';', '<', '>', '(', ')', '$', '`', ' ', '*', '?', '[', '#', '~', '=', '%', '\\',
-        '"', '\'', '\t', '\n',
-    ];
+    let shell_chars = b"|&;<>()$` *?[#~=%'\"\t\n\\";
 
-    let mut command = if alias.command.find(shell_chars).is_some() {
+    let mut command = if alias.command.as_bytes().find_byteset(shell_chars).is_some() {
         // Need to wrap in shell command
         let mut command = std::process::Command::new(shell_path);
         command.arg("-c");
@@ -398,30 +384,26 @@ fn execute_shell_alias(
     command.args(user_args);
 
     if let Some(repo) = repo {
-        if let Some(workdir) = repo.workdir() {
+        if let Some(workdir) = repo.work_dir() {
             command.current_dir(workdir);
-            if let Ok(cur_dir) = std::env::current_dir() {
-                if let Ok(prefix) = cur_dir.strip_prefix(workdir) {
-                    if !cur_dir.starts_with(repo.path()) {
-                        let mut prefix = prefix.as_os_str().to_os_string();
-                        if !prefix.is_empty() {
-                            prefix.push("/");
-                        }
-                        command.env("GIT_PREFIX", &prefix);
-                    }
+            if let Some(Ok(prefix)) = repo.prefix() {
+                let mut prefix = prefix.into_os_string();
+                if !prefix.is_empty() {
+                    prefix.push("/");
                 }
+                command.env("GIT_PREFIX", prefix);
             }
-            if let Ok(rel_dir) = repo.path().strip_prefix(workdir) {
+            if let Ok(rel_dir) = repo.git_dir().strip_prefix(workdir) {
                 if rel_dir == PathBuf::from(".git") {
                     command.env_remove("GIT_DIR");
                 } else {
                     command.env("GIT_DIR", rel_dir);
                 }
             } else {
-                command.env("GIT_DIR", repo.path());
+                command.env("GIT_DIR", repo.git_dir());
             }
         } else {
-            command.env("GIT_DIR", repo.path());
+            command.env("GIT_DIR", repo.git_dir());
         }
     }
 
@@ -472,15 +454,15 @@ fn execute_stgit_alias(
             {
                 execute_command(command, argv, color_choice)
             } else if aliases.contains_key(resolved_cmd_name) {
-                Err(anyhow!("Recursive alias `{}`", alias.name))
+                Err(anyhow!("recursive alias `{}`", alias.name))
             } else {
                 Err(anyhow!(
-                    "Bad alias for `{}`: `{resolved_cmd_name}` is not a stg command",
+                    "bad alias for `{}`: `{resolved_cmd_name}` is not a stg command",
                     alias.name,
                 ))
             }
         }
-        Err(reason) => Err(anyhow!("Bad alias for `{}`: {reason}", alias.name)),
+        Err(reason) => Err(anyhow!("bad alias for `{}`: {reason}", alias.name)),
     };
 
     exit_with_result(result, color_choice)
@@ -494,20 +476,21 @@ fn execute_stgit_alias(
 ///
 /// N.B. the outcome of this alias search depends on the current directory and thus
 /// depends on -C options having been previously processed.
-pub(crate) fn get_aliases() -> Result<(alias::Aliases, Option<git2::Repository>)> {
-    let maybe_repo = git2::Repository::open_from_env().ok();
-    maybe_repo
-        .as_ref()
-        .map_or_else(git2::Config::open_default, git2::Repository::config)
-        .map_or_else(
-            |_| Ok(alias::get_default_aliases()),
-            |config| {
-                alias::get_aliases(&config, |name| {
-                    STGIT_COMMANDS.iter().any(|command| command.name == name) || name == "help"
-                })
-            },
-        )
-        .map(|aliases| (aliases, maybe_repo))
+pub(crate) fn get_aliases() -> Result<(alias::Aliases, Option<gix::Repository>)> {
+    let maybe_repo = gix::Repository::open().ok();
+    let maybe_config = maybe_repo.as_ref().map(|repo| repo.config_snapshot());
+    let config_file = maybe_config.as_ref().map(|snapshot| snapshot.plumbing());
+    let global_config_file;
+    let config_file = if let Some(config_file) = config_file {
+        Some(config_file)
+    } else {
+        global_config_file = gix::config::File::from_globals().ok();
+        global_config_file.as_ref()
+    };
+    let aliases = alias::get_aliases(config_file, |name| {
+        STGIT_COMMANDS.iter().any(|command| command.name == name) || name == "help"
+    })?;
+    Ok((aliases, maybe_repo))
 }
 
 /// Print user-facing message to stderr.
