@@ -3,18 +3,8 @@
 //! Execute commands with git, the stupid content tracker.
 //!
 //! Each function in this module calls-out to a specific git command that is useful to
-//! StGit. This module exists to overcome limitations of libgit2, including:
-//!
-//! - libgit2 behavior is not automatically affected by git configuration. For
-//!   operations where StGit wants to be use/respect git configuration, using git
-//!   directly is a fast-path to correct behavior.
-//! - libgit2 [`does not always apply patches correctly`]. This is essential behavior
-//!   for StGit (but perhaps not so much for other libgit2 applications?).
-//! - libgit2 is missing some behaviors that StGit requires. For example, StGit needs to
-//!   support the full breadth of commit signing. Running `git commit-tree -s` buys all
-//!   this behavior without having to cobble it together with yet more rust dependencies.
-//!
-//! [`does not always apply patches correctly`]: https://github.com/libgit2/libgit2/issues/5717
+//! StGit. This module originally existed to overcome limitations of libgit2, but
+//! remains until gitoxide can replace its behaviors.
 
 mod command;
 pub(crate) mod diff;
@@ -40,18 +30,17 @@ use self::{
     status::{StatusOptions, Statuses},
     version::StupidVersion,
 };
-use crate::signature::TimeExtended;
 
 pub(crate) trait Stupid<'repo, 'index> {
     /// Get StupidContext for running stupid commands.
     fn stupid(&'repo self) -> StupidContext<'repo, 'index>;
 }
 
-impl<'repo, 'index> Stupid<'repo, 'index> for git2::Repository {
+impl<'repo, 'index> Stupid<'repo, 'index> for gix::Repository {
     fn stupid(&'repo self) -> StupidContext<'repo, 'index> {
         StupidContext {
-            git_dir: Some(self.path()),
-            work_dir: self.workdir(),
+            git_dir: Some(self.git_dir()),
+            work_dir: self.work_dir(),
             index_path: None,
             git_version: RefCell::new(None::<StupidVersion>),
         }
@@ -104,13 +93,20 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
         command
     }
 
-    fn git_in_work_root(&self) -> Command {
-        let mut command = self.git();
-        command.current_dir(
-            self.work_dir
-                .expect("work_dir is required for this command"),
-        );
-        command
+    fn git_in_work_root(&self) -> Result<Command> {
+        let mut command = Command::new("git");
+        let work_dir = self
+            .work_dir
+            .expect("work_dir is required for this command");
+        command.current_dir(work_dir);
+        if let Some(git_dir) = self.git_dir {
+            command.env("GIT_DIR", git_dir.canonicalize()?);
+        }
+        command.env("GIT_WORK_TREE", work_dir.canonicalize()?);
+        if let Some(index_path) = self.index_path {
+            command.env("GIT_INDEX_FILE", index_path.canonicalize()?);
+        }
+        Ok(command)
     }
 
     fn at_least_version(&self, version: &StupidVersion) -> Result<bool> {
@@ -127,7 +123,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
 
     /// Apply a patch (diff) to the specified index using `git apply --cached`.
     pub(crate) fn apply_to_index(&self, diff: &[u8]) -> Result<()> {
-        self.git_in_work_root()
+        self.git_in_work_root()?
             .args(["apply", "--cached"]) // TODO: use --recount?
             .stdout(Stdio::null())
             .in_and_out(diff)?
@@ -144,7 +140,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
         directory: Option<&Path>,
         context_lines: Option<usize>,
     ) -> Result<()> {
-        let mut command = self.git_in_work_root();
+        let mut command = self.git_in_work_root()?;
         command.args(["apply", "--index"]);
         if reject {
             command.arg("--reject");
@@ -176,8 +172,8 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Returns `true` if the patch application is successful, `false` otherwise.
     pub(crate) fn apply_treediff_to_index(
         &self,
-        tree1: git2::Oid,
-        tree2: git2::Oid,
+        tree1: gix::ObjectId,
+        tree2: gix::ObjectId,
         want_3way: bool,
     ) -> Result<bool> {
         if tree1 == tree2 {
@@ -193,7 +189,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
             .stdout(Stdio::piped())
             .spawn_git()?;
 
-        let mut apply_cmd = self.git_in_work_root();
+        let mut apply_cmd = self.git_in_work_root()?;
         apply_cmd.args(["apply", "--cached"]);
         if want_3way && self.at_least_version(&StupidVersion::new(2, 32, 0))? {
             apply_cmd.arg("--3way");
@@ -211,8 +207,8 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Apply path limited diff between to trees to specified index.
     pub(crate) fn apply_pathlimited_treediff_to_index<SpecIter, SpecArg>(
         &self,
-        tree1: git2::Oid,
-        tree2: git2::Oid,
+        tree1: gix::ObjectId,
+        tree2: gix::ObjectId,
         want_3way: bool,
         pathspecs: SpecIter,
     ) -> Result<bool>
@@ -242,7 +238,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
             return Ok(true);
         }
 
-        let mut apply_cmd = self.git_in_work_root();
+        let mut apply_cmd = self.git_in_work_root()?;
         apply_cmd.args(["apply", "--cached"]);
         if want_3way && self.at_least_version(&StupidVersion::new(2, 32, 0))? {
             apply_cmd.arg("--3way");
@@ -268,8 +264,8 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Returns `true` if the patch application is successful, `false` otherwise.
     pub(crate) fn apply_treediff_to_worktree_and_index<SpecIter, SpecArg>(
         &self,
-        tree1: git2::Oid,
-        tree2: git2::Oid,
+        tree1: gix::ObjectId,
+        tree2: gix::ObjectId,
         pathspecs: Option<SpecIter>,
     ) -> Result<bool>
     where
@@ -305,7 +301,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
         }
 
         let apply_output = self
-            .git_in_work_root()
+            .git_in_work_root()?
             .args(["apply", "--index", "--3way"])
             .stdin(Stdio::piped())
             .stdout(Stdio::null())
@@ -322,7 +318,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
 
     /// Copy branch
     ///
-    /// Copies branch ref, reflog, and 'branch.<name>' config sections.
+    /// Copies branch ref, reflog, and `branch.<name>` config sections.
     pub(crate) fn branch_copy(
         &self,
         old_branchname: Option<&str>,
@@ -344,7 +340,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Move branch
     ///
     /// Moves branch ref, moves reflog, updates HEAD, and
-    /// renames 'branch.<name>' config sections.
+    /// renames `branch.<name>` config sections.
     pub(crate) fn branch_move(
         &self,
         old_branchname: Option<&str>,
@@ -378,15 +374,17 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Create a commit for the specified tree id using `git commit-tree`.
     ///
     /// The newly created commit id is returned.
-    pub(crate) fn commit_tree(
+    pub(crate) fn commit_tree<'a>(
         &self,
-        author: &git2::Signature,
-        committer: &git2::Signature,
+        author: impl Into<gix::actor::SignatureRef<'a>>,
+        committer: impl Into<gix::actor::SignatureRef<'a>>,
         message: &[u8],
-        tree_id: git2::Oid,
-        parent_ids: impl IntoIterator<Item = git2::Oid>,
+        tree_id: gix::ObjectId,
+        parent_ids: impl IntoIterator<Item = gix::ObjectId>,
         gpgsign: bool,
-    ) -> Result<git2::Oid> {
+    ) -> Result<gix::ObjectId> {
+        let author = author.into();
+        let committer = committer.into();
         let mut command = self.git();
         command.arg("commit-tree").arg(tree_id.to_string());
         for parent_id in parent_ids {
@@ -396,19 +394,19 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
             command.arg("-S");
         }
         let author_name = author
-            .name_bytes()
+            .name
             .to_os_str()
             .context("setting GIT_AUTHOR_NAME from author name")?;
         let author_email = author
-            .email_bytes()
+            .email
             .to_os_str()
             .context("setting GIT_AUTHOR_EMAIL from author email")?;
         let committer_name = committer
-            .name_bytes()
+            .name
             .to_os_str()
             .context("setting GIT_COMMITTER_NAME from author name")?;
         let committer_email = committer
-            .email_bytes()
+            .email
             .to_os_str()
             .context("setting GIT_COMMITTER_EMAIL from author email")?;
 
@@ -418,8 +416,14 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
             .env("GIT_COMMITTER_NAME", committer_name)
             .env("GIT_COMMITTER_EMAIL", committer_email)
             // TODO: re-encode dates?
-            .env("GIT_AUTHOR_DATE", author.epoch_time_string())
-            .env("GIT_COMMITTER_DATE", committer.epoch_time_string())
+            .env(
+                "GIT_AUTHOR_DATE",
+                author.time.format(gix::date::time::format::RAW),
+            )
+            .env(
+                "GIT_COMMITTER_DATE",
+                committer.time.format(gix::date::time::format::RAW),
+            )
             .stdin(Stdio::piped())
             .stdout(Stdio::piped())
             .stderr(Stdio::piped())
@@ -502,7 +506,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     }
 
     /// Generate diff between specified tree and the working tree or index with `git diff-index`.
-    pub(crate) fn diff_index(&self, tree_id: git2::Oid) -> Result<Vec<u8>> {
+    pub(crate) fn diff_index(&self, tree_id: gix::ObjectId) -> Result<Vec<u8>> {
         let output = self
             .git()
             .args(["diff-index", "-p", "--full-index"])
@@ -515,7 +519,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Get file names that differ between tree and index.
     pub(crate) fn diff_index_names(
         &self,
-        tree_id: git2::Oid,
+        tree_id: gix::ObjectId,
         relative: Option<&Path>,
     ) -> Result<Vec<u8>> {
         let mut command = self.git();
@@ -540,9 +544,9 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
         Ok(output.stdout)
     }
 
-    /// Diff tree with index returning a bool indicating whether they match.
-    pub(crate) fn diff_index_quiet(&self, tree_id: git2::Oid) -> Result<bool> {
-        Ok(self
+    /// Diff tree with index returning a bool indicating whether they differ.
+    pub(crate) fn diff_index_quiet(&self, tree_id: gix::ObjectId) -> Result<bool> {
+        let no_diff = self
             .git()
             .args(["diff-index", "--quiet", "--cached"])
             .arg(tree_id.to_string())
@@ -550,11 +554,16 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
             .stderr(Stdio::null())
             .output_git()?
             .status
-            .success())
+            .success();
+        Ok(!no_diff)
     }
 
     /// Get names of files that differ between two trees.
-    pub(crate) fn diff_tree_files(&self, tree1: git2::Oid, tree2: git2::Oid) -> Result<DiffFiles> {
+    pub(crate) fn diff_tree_files(
+        &self,
+        tree1: gix::ObjectId,
+        tree2: gix::ObjectId,
+    ) -> Result<DiffFiles> {
         self.git()
             .args(["diff-tree", "-r", "--name-only", "-z"])
             .args([tree1.to_string(), tree2.to_string()])
@@ -566,8 +575,8 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Interactive diff-tree (for 'stg files').
     pub(crate) fn diff_tree_files_status(
         &self,
-        tree1: git2::Oid,
-        tree2: git2::Oid,
+        tree1: gix::ObjectId,
+        tree2: gix::ObjectId,
         stat: bool,
         name_only: bool,
         use_color: bool,
@@ -594,8 +603,8 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Generate diff between two trees using `git diff-tree -p`.
     pub(crate) fn diff_tree_patch<SpecIter, SpecArg, OptIter, OptArg>(
         &self,
-        tree1: git2::Oid,
-        tree2: git2::Oid,
+        tree1: gix::ObjectId,
+        tree2: gix::ObjectId,
         pathspecs: Option<SpecIter>,
         use_color: bool,
         diff_opts: OptIter,
@@ -663,7 +672,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Show log in gitk
     pub(crate) fn gitk<SpecIter, SpecArg>(
         &self,
-        commit_id: git2::Oid,
+        commit_id: gix::ObjectId,
         pathspecs: Option<SpecIter>,
     ) -> Result<()>
     where
@@ -716,7 +725,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Interactively show log
     pub(crate) fn log<SpecIter, SpecArg>(
         &self,
-        commit_id: git2::Oid,
+        commit_id: gix::ObjectId,
         pathspecs: Option<SpecIter>,
         num_commits: Option<usize>,
         use_color: bool,
@@ -727,7 +736,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
         SpecIter: IntoIterator<Item = SpecArg>,
         SpecArg: AsRef<OsStr>,
     {
-        let mut command = self.git_in_work_root();
+        let mut command = self.git_in_work_root()?;
         command.arg("log");
         if let Some(n) = num_commits {
             command.arg(format!("-{n}"));
@@ -858,14 +867,36 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
         Ok(num_patches)
     }
 
+    pub(crate) fn merge_bases(
+        &self,
+        id0: gix::ObjectId,
+        id1: gix::ObjectId,
+    ) -> Result<Vec<gix::ObjectId>> {
+        let output = self
+            .git()
+            .args(["merge-base", "--all"])
+            .args([id0.to_string(), id1.to_string()])
+            .output_git()?
+            .require_success("merge-base --all")?;
+        let mut oids: Vec<gix::ObjectId> = Vec::new();
+        for line in output
+            .stdout
+            .split_str("\n")
+            .filter(|line| !line.is_empty())
+        {
+            oids.push(parse_oid(line)?);
+        }
+        Ok(oids)
+    }
+
     /// Perform three-way merge with `git merge-recursive`.
     ///
     /// Returns `true` if the merge was successful, `false` otherwise.
     pub(crate) fn merge_recursive(
         &self,
-        base_tree_id: git2::Oid,
-        our_tree_id: git2::Oid,
-        their_tree_id: git2::Oid,
+        base_tree_id: gix::ObjectId,
+        our_tree_id: gix::ObjectId,
+        their_tree_id: gix::ObjectId,
     ) -> Result<bool> {
         let output = self
             .git()
@@ -891,9 +922,9 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Perform three-way merge, with optional auto-resolution of conflicts with `git merge-tool`.
     pub(crate) fn merge_recursive_or_mergetool(
         &self,
-        base_tree_id: git2::Oid,
-        our_tree_id: git2::Oid,
-        their_tree_id: git2::Oid,
+        base_tree_id: gix::ObjectId,
+        our_tree_id: gix::ObjectId,
+        their_tree_id: gix::ObjectId,
         use_mergetool: bool,
     ) -> Result<bool> {
         if self.merge_recursive(base_tree_id, our_tree_id, their_tree_id)? {
@@ -918,7 +949,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     }
 
     /// Copy notes from one object to another using `git notes copy`.
-    pub(crate) fn notes_copy(&self, from_oid: git2::Oid, to_oid: git2::Oid) -> Result<()> {
+    pub(crate) fn notes_copy(&self, from_oid: gix::ObjectId, to_oid: gix::ObjectId) -> Result<()> {
         self.git()
             .args(["notes", "copy"])
             .arg(from_oid.to_string())
@@ -930,7 +961,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     }
 
     /// Read content of a tree into specified index using `git read-tree`.
-    pub(crate) fn read_tree(&self, tree_id: git2::Oid) -> Result<()> {
+    pub(crate) fn read_tree(&self, tree_id: gix::ObjectId) -> Result<()> {
         self.git()
             .arg("read-tree")
             .arg(tree_id.to_string())
@@ -943,8 +974,8 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Checkout tree to working tree using `git read-tree`.
     pub(crate) fn read_tree_checkout(
         &self,
-        old_tree_id: git2::Oid,
-        new_tree_id: git2::Oid,
+        old_tree_id: gix::ObjectId,
+        new_tree_id: gix::ObjectId,
     ) -> Result<()> {
         self.git()
             .args([
@@ -962,7 +993,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     }
 
     /// Hard checkout tree to working tree using `git read-tree`.
-    pub(crate) fn read_tree_checkout_hard(&self, tree_id: git2::Oid) -> Result<()> {
+    pub(crate) fn read_tree_checkout_hard(&self, tree_id: gix::ObjectId) -> Result<()> {
         self.git()
             .args(["read-tree", "--reset", "-u"])
             .arg(tree_id.to_string())
@@ -985,10 +1016,10 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Get list of revisions using `git rev-list`.
     pub(crate) fn rev_list<SpecIter, SpecArg>(
         &self,
-        base: git2::Oid,
-        top: git2::Oid,
+        base: gix::ObjectId,
+        top: gix::ObjectId,
         pathspecs: Option<SpecIter>,
-    ) -> Result<Vec<git2::Oid>>
+    ) -> Result<Vec<gix::ObjectId>>
     where
         SpecIter: IntoIterator<Item = SpecArg>,
         SpecArg: AsRef<OsStr>,
@@ -1002,7 +1033,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
         }
 
         let output = command.output_git()?.require_success("rev-list")?;
-        let mut oids: Vec<git2::Oid> = Vec::new();
+        let mut oids: Vec<gix::ObjectId> = Vec::new();
         for line in output
             .stdout
             .split_str("\n")
@@ -1025,7 +1056,27 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
         assert_eq!(last, Some(b'\n'));
         match stdout.into_os_string() {
             Ok(cdup) => Ok(cdup),
-            Err(_) => Err(anyhow!("Could not convert cdup to path")),
+            Err(_) => Err(anyhow!("could not convert cdup to path")),
+        }
+    }
+
+    pub(crate) fn rev_parse_symbolic_full_name(&self, name: &str) -> Result<Option<String>> {
+        let output = self
+            .git()
+            .args([
+                "rev-parse",
+                "--symbolic-full-name",
+                "--verify",
+                "--end-of-options",
+                name,
+            ])
+            .output_git()?
+            .require_success("rev-parse --symbolic-full-name")?;
+        let full_name = output.stdout.to_str()?.trim_end();
+        if full_name.is_empty() {
+            Ok(None)
+        } else {
+            Ok(Some(full_name.to_string()))
         }
     }
 
@@ -1058,7 +1109,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     /// Show objects using `git show`.
     pub(crate) fn show<SpecIter, SpecArg, OptIter, OptArg>(
         &self,
-        oids: impl IntoIterator<Item = git2::Oid>,
+        oids: impl IntoIterator<Item = gix::ObjectId>,
         pathspecs: Option<SpecIter>,
         stat: bool,
         use_color: bool,
@@ -1100,7 +1151,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     }
 
     /// Show object with custom pretty format.
-    pub(crate) fn show_pretty(&self, oid: git2::Oid, pretty_format: &str) -> Result<Vec<u8>> {
+    pub(crate) fn show_pretty(&self, oid: gix::ObjectId, pretty_format: &str) -> Result<Vec<u8>> {
         let output = self
             .git()
             .args(["show", "--no-patch"])
@@ -1218,7 +1269,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
         SpecArg: AsRef<OsStr> + Send,
     {
         let mut child = self
-            .git_in_work_root()
+            .git_in_work_root()?
             .args([
                 "update-index",
                 "--remove",
@@ -1291,7 +1342,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
                 Err(anyhow!("`{user_cmd_str}` failed"))
             }
         } else {
-            Err(anyhow!("User-provided fetchcmd is empty"))
+            Err(anyhow!("user-provided fetchcmd is empty"))
         }
     }
 
@@ -1325,12 +1376,12 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
                 Err(anyhow!("`{user_cmd_str}` failed"))
             }
         } else {
-            Err(anyhow!("User-provided pullcmd is empty"))
+            Err(anyhow!("user-provided pullcmd is empty"))
         }
     }
 
     /// Run user-provided rebase command.
-    pub(crate) fn user_rebase(&self, user_cmd_str: &str, target: git2::Oid) -> Result<()> {
+    pub(crate) fn user_rebase(&self, user_cmd_str: &str, target: gix::ObjectId) -> Result<()> {
         let mut args = user_cmd_str.split(|c: char| c.is_ascii_whitespace());
         if let Some(command_name) = args.next() {
             let mut command = Command::new(command_name);
@@ -1354,7 +1405,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
                 Err(anyhow!("`{user_cmd_str}` failed"))
             }
         } else {
-            Err(anyhow!("User-provided rebasecmd is empty"))
+            Err(anyhow!("user-provided rebasecmd is empty"))
         }
     }
 
@@ -1374,7 +1425,7 @@ impl<'repo, 'index> StupidContext<'repo, 'index> {
     }
 
     /// Write tree object from content of specified index using `git write-tree`.
-    pub(crate) fn write_tree(&self) -> Result<git2::Oid> {
+    pub(crate) fn write_tree(&self) -> Result<gix::ObjectId> {
         let output = self
             .git()
             .arg("write-tree")
