@@ -18,7 +18,7 @@ test_expect_success 'Create a branch (and switch to it)' '
 
 test_expect_success 'Attempt to delete branch with patches' '
     command_error stg branch --delete master 2>err &&
-    grep -e "Delete not permitted: the series still contains patches" err
+    grep -e "delete not permitted: the series still contains patches" err
 '
 
 test_expect_success 'Force delete branch with patches' '
@@ -41,14 +41,14 @@ test_expect_success 'Make sure the branch files were deleted' '
 
 test_expect_success 'Attempt to delete current branch' '
     command_error stg branch --delete $(stg branch) 2>err &&
-    grep -e "Cannot delete the current branch" err
+    grep -e "cannot delete the current branch" err
 '
 
 test_expect_success 'Invalid num args to delete' '
     general_error stg branch --delete 2>err &&
-    grep -e "The following required arguments were not provided" err &&
+    grep -e "the following required arguments were not provided" err &&
     general_error stg branch --delete foo extra 2>err &&
-    grep -e "Found argument .extra. which wasn.t expected" err
+    grep -e "unexpected argument .extra." err
 '
 
 test_expect_success 'Create a non-StGit branch and delete it' '
@@ -58,7 +58,7 @@ test_expect_success 'Create a non-StGit branch and delete it' '
 
 test_expect_success 'Delete a nonexistent branch' '
    command_error stg branch --delete bar 2>err &&
-   grep -e "Branch \`bar\` not found" err
+   grep -e "branch \`bar\` not found" err
 '
 
 test_done
