@@ -1,39 +1,33 @@
 #!/bin/sh
-#
-# Copyright (c) 2009 David Kågedal
-#
 
-test_description='Exercise pushing patches with --set-tree.'
+# Copyright (c) 2009 David Kågedal
+
+test_description='Exercise pushing patches with --set-tree'
 
 . ./test-lib.sh
 
-test_expect_success \
-    'Create initial patches' '
-    stg init &&
+test_expect_success 'Create initial patches' '
     stg new A -m A &&
-    echo hello world > a &&
-    git add a &&
+    echo hello world >a &&
+    stg add a &&
     stg refresh
     stg new B -m B &&
-    echo HELLO WORLD > a &&
+    echo HELLO WORLD >a &&
     stg refresh
 '
 
-test_expect_success \
-    'Back up and create a partial patch' '
+test_expect_success 'Back up and create a partial patch' '
     stg pop &&
     stg new C -m C &&
-    echo hello WORLD > a &&
+    echo hello WORLD >a &&
     stg refresh
 '
 
-test_expect_success \
-    'Reapply patch B' '
+test_expect_success 'Reapply patch B' '
     stg push --set-tree B
 '
 
-test_expect_success \
-    'Compare results' '
+test_expect_success 'Compare results' '
     stg pop -a &&
     stg push &&
     test "$(echo $(cat a))" = "hello world" &&
