@@ -166,7 +166,7 @@ test_expect_success 'Import with author options' '
 test_expect_success 'Import with bad author_date option' '
     stg delete --top &&
     general_error stg import --authdate "a long time ago" some.patch 2>err &&
-    grep -e "Invalid date \`a long time ago\`" err
+    grep -e "invalid date \`a long time ago\`" err
 '
 
 test_expect_success 'Import from stdin' '
@@ -242,7 +242,7 @@ test_expect_success 'Attempt url' '
 
 test_expect_success 'Too many arguments' '
     general_error stg import some.patch some.patch 2>err &&
-    grep -e "Found argument .some\.patch. which wasn.t expected" err
+    grep -e "unexpected argument .some\.patch." err
 '
 
 test_done
