@@ -4,18 +4,13 @@ test_description='Test import from emails'
 
 . ./test-lib.sh
 
-test_expect_success \
-    'Initialize the StGit repository' \
-    '
+test_expect_success 'Initialize the StGit repository' '
     cp "$TEST_DIRECTORY"/t1800/foo.txt . &&
     stg add foo.txt &&
-    git commit -a -m "initial version" &&
-    stg init
-    '
+    git commit -a -m "initial version"
+'
 
-test_expect_success \
-    'Apply a patch from an 8bit-encoded e-mail' \
-    '
+test_expect_success 'Apply a patch from an 8bit-encoded e-mail' '
     stg import -m --message-id "$TEST_DIRECTORY"/t1801/email-8bit &&
     [ $(git cat-file -p $(stg id) \
         | grep -c "tree 030be42660323ff2a1958f9ee79589a4f3fbee2f") = 1 ] &&
@@ -24,11 +19,9 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 1 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Apply a patch from an 8bit-encoded e-mail url' \
-    '
+test_expect_success 'Apply a patch from an 8bit-encoded e-mail url' '
     stg import -u -m "file://$TEST_DIRECTORY"/t1801/email-8bit &&
     [ $(git cat-file -p $(stg id) \
         | grep -c "tree 030be42660323ff2a1958f9ee79589a4f3fbee2f") = 1 ] &&
@@ -37,11 +30,9 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Apply a patch from an 8bit-encoded e-mail with CRLF endings' \
-    '
+test_expect_success 'Apply a patch from an 8bit-encoded e-mail with CRLF endings' '
     cat "$TEST_DIRECTORY"/t1801/email-8bit | append_cr |
     stg import -m &&
     [ $(git cat-file -p $(stg id) \
@@ -51,13 +42,11 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Apply e-mail with CRLF endings and --keep-cr' \
-    '
+test_expect_success 'Apply e-mail with CRLF endings and --keep-cr' '
     stg new -m foo-with-crlf &&
-    cat foo.txt | append_cr > foo-crlf.txt &&
+    cat foo.txt | append_cr >foo-crlf.txt &&
     mv foo-crlf.txt foo.txt &&
     stg refresh &&
     cat "$TEST_DIRECTORY"/t1801/email-8bit | append_cr |
@@ -69,12 +58,10 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Apply a patch from latin1-encoded email specifying utf-8 charset' \
-    '
-    iconv -f UTF-8 -t LATIN1 "$TEST_DIRECTORY"/t1801/email-8bit > email-latin1 &&
+test_expect_success 'Apply a patch from latin1-encoded email specifying utf-8 charset' '
+    iconv -f UTF-8 -t LATIN1 "$TEST_DIRECTORY"/t1801/email-8bit >email-latin1 &&
     stg import -m email-latin1 &&
     [ $(git cat-file -p $(stg id) \
         | grep -c "tree cf0f9884fdb30bca14d2411e1711f6ae413c9213") = 1 ] &&
@@ -83,11 +70,9 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Apply a patch from email with quoted "From" header' \
-    '
+test_expect_success 'Apply a patch from email with quoted "From" header' '
     stg import -m "$TEST_DIRECTORY"/t1801/email-quoted-from &&
     [ $(git cat-file -p $(stg id) \
         | grep -c "tree 030be42660323ff2a1958f9ee79589a4f3fbee2f") = 1 ] &&
@@ -96,11 +81,9 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Apply a patch from a QP-encoded e-mail' \
-    '
+test_expect_success 'Apply a patch from a QP-encoded e-mail' '
     stg import -m "$TEST_DIRECTORY"/t1801/email-qp &&
     [ $(git cat-file -p $(stg id) \
         | grep -c "tree 030be42660323ff2a1958f9ee79589a4f3fbee2f") = 1 ] &&
@@ -109,11 +92,9 @@ test_expect_success \
     [ $(git cat-file -p $(stg id) \
         | grep -c "Message-Id: <20061111105814.23209.46952.stgit@localhost>") = 0 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Apply several patches from an mbox file' \
-    '
+test_expect_success 'Apply several patches from an mbox file' '
     test_config stgit.import.message-id "no" &&
     stg import -M --message-id "$TEST_DIRECTORY"/t1801/email-mbox &&
     [ $(git cat-file -p $(stg id change-1) \
@@ -135,11 +116,9 @@ test_expect_success \
     [ $(git cat-file -p $(stg id change-3-colon) \
         | grep -c "Message-Id: <20061111114527.31778.45876.stgit@localhost>") = 1 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Apply several patches from an mbox url' \
-    '
+test_expect_success 'Apply several patches from an mbox url' '
     test_config stgit.import.message-id "yes" &&
     stg import -u -M "file://$TEST_DIRECTORY"/t1801/email-mbox &&
     [ $(git cat-file -p $(stg id change-1) \
@@ -161,19 +140,15 @@ test_expect_success \
     [ $(git cat-file -p $(stg id change-3-colon) \
         | grep -c "Message-Id: <20061111114527.31778.45876.stgit@localhost>") = 1 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Import patches from mbox with duplicate subjects' \
-    '
+test_expect_success 'Import patches from mbox with duplicate subjects' '
     stg import -M "$TEST_DIRECTORY"/t1801/email-mbox-same-subject &&
     test "$(echo $(stg series --noprefix --applied))" = "my-patch my-patch-1 my-patch-2" &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Apply several patches from an mbox file from stdin' \
-    '
+test_expect_success 'Apply several patches from an mbox file from stdin' '
     test_config stgit.import.message-id "off" &&
     cat "$TEST_DIRECTORY"/t1801/email-mbox | stg import -M &&
     [ $(git cat-file -p $(stg id change-1) \
@@ -195,11 +170,9 @@ test_expect_success \
     [ $(git cat-file -p $(stg id change-3-colon) \
         | grep -c "Message-Id: <20061111114527.31778.45876.stgit@localhost>") = 0 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Apply several patches from an mbox file with CRLF line endings' \
-    '
+test_expect_success 'Apply several patches from an mbox file with CRLF line endings' '
     cat "$TEST_DIRECTORY"/t1801/email-mbox | append_cr |
     stg import -M &&
     [ $(git cat-file -p $(stg id change-1) \
@@ -215,14 +188,13 @@ test_expect_success \
     [ $(git cat-file -p $(stg id change-3-colon) \
         | grep -c "author Inge Ström <inge@power.com>") = 1 ] &&
     stg delete ..
-    '
+'
 
-test_expect_success \
-    'Import from git format-patch output' '
+test_expect_success 'Import from git format-patch output' '
     (
         test_create_repo upstream &&
         cd upstream &&
-        echo "something" > some.txt &&
+        echo "something" >some.txt &&
         git add some.txt &&
         git commit -m "Add something"
         stg init
@@ -230,9 +202,9 @@ test_expect_success \
     (
         git clone upstream downstream
         cd downstream &&
-        echo "else µ" >> some.txt &&
+        echo "else µ" >>some.txt &&
         git commit -a --author "Éd <ed@example.com>" -m "something else" &&
-        git format-patch --stdout HEAD~1 > ../downstream.mbox
+        git format-patch --stdout HEAD~1 >../downstream.mbox
     ) &&
     (
         cd upstream &&
@@ -240,6 +212,6 @@ test_expect_success \
         stg top | grep "something-else" &&
         grep "else µ" some.txt
     )
-    '
+'
 
 test_done
