@@ -36,12 +36,12 @@ test_expect_success 'Create a branch with empty patches' '
 
 test_expect_success 'Attempt sync with current branch' '
     command_error stg sync -B foo 2>err &&
-    grep -e "Cannot synchroni[sz]e with the current branch" err
+    grep -e "cannot synchroni[sz]e with the current branch" err
 '
 
 test_expect_success 'Attempt sync without remote branch or series' '
     general_error stg sync -a 2>err &&
-    grep -e "The following required arguments were not provided" err
+    grep -e "the following required arguments were not provided" err
 '
 
 test_expect_success 'Attempt apply top patch without any applied' '
@@ -55,7 +55,7 @@ test_expect_success 'Attempt to sync patch not in ref branch' '
     stg new -m p4 &&
     test_when_finished "stg delete p4" &&
     command_error stg sync -B master 2>err &&
-    grep -e "No common patches to synchronize" err
+    grep -e "no common patches to synchronize" err
 '
 
 test_expect_success 'Synchronise second patch with the master branch' '
@@ -74,7 +74,7 @@ test_expect_success 'Synchronise the first two patches with the master branch' '
 '
 
 test_expect_success 'Synchronise all the patches with the exported series' '
-    stg sync -s patches-master/series -a &&
+    stg sync -S patches-master/series -a &&
     [ "$(echo $(stg series --applied --noprefix))" = "p1 p2 p3" ] &&
     [ "$(echo $(stg series --unapplied --noprefix))" = "" ] &&
     test "$(cat foo1.txt)" = "foo1" &&
@@ -125,7 +125,7 @@ test_expect_success 'Restore the stack status after the failed sync' '
 '
 
 test_expect_success 'Synchronise the third patch with the exported series (to fail)' '
-    conflict stg sync -s patches-master/series p3
+    conflict stg sync -S patches-master/series p3
 '
 
 test_expect_success 'Restore the stack status after the failed sync' '
